==> apgl-0.6.tar.gz: apgl-0.6/apgl/graph/GraphStatistics.py <==
"""
A class to compute a series of graph statistics over a sequence of subgraphs
"""

import numpy
import logging 
from apgl.util.Util import Util
from apgl.graph.GraphUtils import GraphUtils

class GraphStatistics(object):
    def __init__(self):
        self.numVerticesIndex = 0
        self.numEdgesIndex = 1
        self.numDirEdgesIndex = 2
        self.maxComponentSizeIndex = 3
        self.numComponentsIndex = 4
        self.meanComponentSizeIndex = 5
        self.meanDegreeIndex = 6
        self.diameterIndex = 7
        self.effectiveDiameterIndex = 8
        self.densityIndex = 9
        self.powerLawIndex = 10
        self.geodesicDistanceIndex = 11
        self.harmonicGeoDistanceIndex = 12

        self.numStats = 13
        self.q = 0.9
        self.printStep = 5
        self.vectorPrintStep = 1

    def scalarStatistics(self, graph, slowStats=True):
        """
        Find a series of statistics for the given input graph which can be represented
        as scalar values. Return as a vector.
        """
        statsArray = numpy.ones(self.numStats)*-1
        statsArray[self.numVerticesIndex] = graph.getNumVertices()
        statsArray[self.numEdgesIndex] = graph.getNumEdges()
        statsArray[self.numDirEdgesIndex] = graph.getNumDirEdges()
        statsArray[self.densityIndex] = graph.density()

        if graph.isUndirected():
            subComponents = graph.findConnectedComponents()
            statsArray[self.numComponentsIndex] = len(subComponents)

            if len(subComponents) != 0:
                statsArray[self.maxComponentSizeIndex] = len(subComponents[0])
                statsArray[self.meanComponentSizeIndex] = sum([len(x) for x in subComponents])/float(statsArray[self.numComponentsIndex])
            else:
                statsArray[self.maxComponentSizeIndex] = 0
                statsArray[self.meanComponentSizeIndex] = 0 

        if graph.getNumVertices() != 0:
            statsArray[self.meanDegreeIndex] = 2*float(graph.getNumEdges())/graph.getNumVertices()
        else:
            statsArray[self.meanDegreeIndex] = 0
            
        if slowStats:
            P = graph.floydWarshall(False)
            statsArray[self.diameterIndex] = graph.diameter(P=P)
            statsArray[self.effectiveDiameterIndex] = graph.effectiveDiameter(self.q, P=P)
            statsArray[self.powerLawIndex] = graph.fitPowerLaw()[0]
            statsArray[self.geodesicDistanceIndex] = graph.geodesicDistance(P=P)
            statsArray[self.harmonicGeoDistanceIndex] = graph.harmonicGeodesicDistance(P=P)

        return statsArray

    def vectorStatistics(self, graph, treeStats=False):
        """
        Find a series of statistics for the given input graph which can be represented 
        as vector values.
        """
        statsDict = {}

        statsDict["inDegreeDist"] = graph.inDegreeDistribution()
        statsDict["outDegreeDist"] = graph.degreeDistribution()
        logging.debug("Computing hop counts")
        statsDict["hopCount"] = graph.hopCount()
        logging.debug("Computing triangle count")
        statsDict["triangleDist"] = numpy.bincount(graph.triangleSequence())
        
        #Get the distribution of component sizes 
        logging.debug("Finding distribution of component sizes")
        
        if graph.isUndirected(): 
            components = graph.findConnectedComponents()
            statsDict["componentsDist"] = numpy.bincount(numpy.array([len(c) for c in components]))

        #Make sure weight matrix is symmetric
        logging.debug("Computing eigenvalues/vectors")
        W = graph.getWeightMatrix()
        W = (W + W.T)/2
        eigenDistribution, V = numpy.linalg.eig(W)
        i = numpy.argmax(eigenDistribution)
        statsDict["maxEigVector"] = V[:, i]
        statsDict["eigenDist"] = numpy.flipud(numpy.sort(eigenDistribution[eigenDistribution>0]))

        if treeStats:
            logging.debug("Computing statistics on trees")
            trees = graph.findTrees()
            statsDict["treeSizesDist"] = numpy.bincount([len(x) for x in trees])
            treeDepths = [GraphUtils.treeDepth((graph.subgraph(x))) for x in trees]
            statsDict["treeDepthsDist"] = numpy.bincount(treeDepths)

        return statsDict

    def sequenceScalarStats(self, graph, subgraphIndices, slowStats=True):
        """
        Pass in a list of graphs are returns a series of statistics. Each row
        corresponds to the statistics on the subgraph. 
        """

        numGraphs = len(subgraphIndices)
        statsMatrix = numpy.zeros((numGraphs, self.numStats))

        for i in range(numGraphs):
            Util.printIteration(i, self.printStep, numGraphs)
            subgraph = graph.subgraph(subgraphIndices[i])
            statsMatrix[i, :] = self.scalarStatistics(subgraph, slowStats)

        return statsMatrix

    def sequenceVectorStats(self, graph, subgraphIndices, treeStats=False):
        """
        Pass in a list of graphs are returns a series of statistics. Each list
        element is a dict of vector statistics. 
        """

        numGraphs = len(subgraphIndices)
        statsDictList = []

        for i in range(numGraphs):
            Util.printIteration(i, self.vectorPrintStep, numGraphs)
            subgraph = graph.subgraph(subgraphIndices[i])
            statsDictList.append(self.vectorStatistics(subgraph, treeStats))

        return statsDictList

==> apgl-0.6.1.tar.gz: apgl-0.6.1/apgl/graph/GraphStatistics.py <==
"""
A class to compute a series of graph statistics over a sequence of subgraphs
"""

import numpy
import logging 
from apgl.util.Util import Util
from apgl.util.Parameter import Parameter 
from apgl.graph.GraphUtils import GraphUtils
from apgl.graph.AbstractMatrixGraph import AbstractMatrixGraph

class GraphStatistics(object):
    def __init__(self):
        self.numVerticesIndex = 0
        self.numEdgesIndex = 1
        self.numDirEdgesIndex = 2
        self.maxComponentSizeIndex = 3
        self.numComponentsIndex = 4
        self.meanComponentSizeIndex = 5
        self.meanDegreeIndex = 6
        self.diameterIndex = 7
        self.effectiveDiameterIndex = 8
        self.densityIndex = 9
        self.powerLawIndex = 10
        self.geodesicDistanceIndex = 11
        self.harmonicGeoDistanceIndex = 12
        self.geodesicDistMaxCompIndex = 13
        self.meanTreeSizeIndex = 14
        self.meanTreeDepthIndex = 15
        self.numNonSingletonComponentsIndex = 16 

        self.numStats = 17
        self.q = 0.9
        self.printStep = 5
        self.vectorPrintStep = 1

    def scalarStatistics(self, graph, slowStats=True, treeStats=False):
        """
        Find a series of statistics for the given input graph which can be represented
        as scalar values. Return results as a vector.
        """
        #This method is a bit of a mess 
        Parameter.checkClass(graph, AbstractMatrixGraph)
        Parameter.checkBoolean(slowStats)
        statsArray = numpy.ones(self.numStats)*-1
        statsArray[self.numVerticesIndex] = graph.getNumVertices()
        statsArray[self.numEdgesIndex] = graph.getNumEdges()
        statsArray[self.numDirEdgesIndex] = graph.getNumDirEdges()
        statsArray[self.densityIndex] = graph.density()

        if graph.isUndirected():
            subComponents = graph.findConnectedComponents()
            statsArray[self.numComponentsIndex] = len(subComponents)
            
            nonSingletonSubComponents = [c for c in subComponents if len(c) > 1]
            statsArray[self.numNonSingletonComponentsIndex] = len(nonSingletonSubComponents)

            if len(subComponents) != 0:
                statsArray[self.maxComponentSizeIndex] = len(subComponents[0])
                statsArray[self.meanComponentSizeIndex] = sum([len(x) for x in subComponents])/float(statsArray[self.numComponentsIndex])
            else:
                statsArray[self.maxComponentSizeIndex] = 0
                statsArray[self.meanComponentSizeIndex] = 0
                statsArray[self.geodesicDistMaxCompIndex] = 0

        if graph.getNumVertices() != 0:
            statsArray[self.meanDegreeIndex] = 2*float(graph.getNumEdges())/graph.getNumVertices()
        else:
            statsArray[self.meanDegreeIndex] = 0
            
        if slowStats:
            logging.debug("Running Floyd-Warshall")
            P = graph.floydWarshall(False)
            statsArray[self.diameterIndex] = graph.diameter(P=P)
            statsArray[self.effectiveDiameterIndex] = graph.effectiveDiameter(self.q, P=P)
            statsArray[self.powerLawIndex] = graph.fitPowerLaw()[0]
            statsArray[self.geodesicDistanceIndex] = graph.geodesicDistance(P=P)
            statsArray[self.harmonicGeoDistanceIndex] = graph.harmonicGeodesicDistance(P=P)

            if graph.isUndirected() and len(subComponents) != 0:
                statsArray[self.geodesicDistMaxCompIndex] = graph.geodesicDistance(P=P, vertexInds=list(subComponents[0]))

        if treeStats:
            logging.debug("Computing statistics on trees")
            trees = graph.findTrees()
            statsArray[self.meanTreeSizeIndex] = numpy.mean([len(x) for x in trees])
            treeDepths = [GraphUtils.treeDepth((graph.subgraph(x))) for x in trees]
            statsArray[self.meanTreeDepthIndex] = numpy.mean(treeDepths)

        return statsArray

    def vectorStatistics(self, graph, treeStats=False):
        """
        Find a series of statistics for the given input graph which can be represented 
        as vector values.
        """
        Parameter.checkClass(graph, AbstractMatrixGraph)
        Parameter.checkBoolean(treeStats)
        statsDict = {}

        statsDict["inDegreeDist"] = graph.inDegreeDistribution()
        statsDict["outDegreeDist"] = graph.degreeDistribution()
        logging.debug("Computing hop counts")
        statsDict["hopCount"] = graph.hopCount()
        logging.debug("Computing triangle count")
        statsDict["triangleDist"] = numpy.bincount(graph.triangleSequence())
        
        #Get the distribution of component sizes 
        logging.debug("Finding distribution of component sizes")
        
        if graph.isUndirected(): 
            components = graph.findConnectedComponents()
            if len(components) != 0: 
                statsDict["componentsDist"] = numpy.bincount(numpy.array([len(c) for c in components], numpy.int))

        #Make sure weight matrix is symmetric
        logging.debug("Computing eigenvalues/vectors")
        W = graph.getWeightMatrix()
        W = (W + W.T)/2
        eigenDistribution, V = numpy.linalg.eig(W)
        i = numpy.argmax(eigenDistribution)
        statsDict["maxEigVector"] = V[:, i]
        statsDict["eigenDist"] = numpy.flipud(numpy.sort(eigenDistribution[eigenDistribution>0]))

        if treeStats:
            logging.debug("Computing statistics on trees")
            trees = graph.findTrees()
            statsDict["treeSizesDist"] = numpy.bincount([len(x) for x in trees])
            treeDepths = [GraphUtils.treeDepth((graph.subgraph(x))) for x in trees]
            statsDict["treeDepthsDist"] = numpy.bincount(treeDepths)

        return statsDict

    def sequenceScalarStats(self, graph, subgraphIndices, slowStats=True, treeStats=False):
        """
        Pass in a list of graphs are returns a series of statistics. Each row
        corresponds to the statistics on the subgraph. 
        """

        numGraphs = len(subgraphIndices)
        statsMatrix = numpy.zeros((numGraphs, self.numStats))

        for i in range(numGraphs):
            Util.printIteration(i, self.printStep, numGraphs)
            logging.debug("Subgraph size: " + str(len(subgraphIndices[i])))
            subgraph = graph.subgraph(subgraphIndices[i])
            statsMatrix[i, :] = self.scalarStatistics(subgraph, slowStats, treeStats)

        return statsMatrix

    def sequenceVectorStats(self, graph, subgraphIndices, treeStats=False):
        """
        Pass in a list of graphs are returns a series of statistics. Each list
        element is a dict of vector statistics. 
        """

        numGraphs = len(subgraphIndices)
        statsDictList = []

        for i in range(numGraphs):
            Util.printIteration(i, self.vectorPrintStep, numGraphs)
            subgraph = graph.subgraph(subgraphIndices[i])
            statsDictList.append(self.vectorStatistics(subgraph, treeStats))

        return statsDictList

    def sequenceClustering(self, graph, subgraphIndices, clusterFunc, maxComponent=True):
        """
        Take a graph and a sequence of indices corresponding to subgraphs and
        compute some clusters indices for each one. 
        """
        numGraphs = len(subgraphIndices)
        clusterList = []

        for i in range(numGraphs):
            Util.printIteration(i, self.vectorPrintStep, numGraphs)
            subgraph = graph.subgraph(subgraphIndices[i])

            if maxComponent:
                subComponents = subgraph.findConnectedComponents()
                subgraph = subgraph.subgraph(subComponents[-1])

            clusterList.append(clusterFunc(subgraph))

        return clusterList
